#define DATE "20000701"
#define VERSION "1.1"

#include <linux/module.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/i2c.h>

#include "fi1216mk2.h"

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,13)
#define THIS_MODULE NULL
#endif

#define TUNER_SYSCTL_FREQ 1000

int debug =  0; /* insmod parameter */
int type  =  -1; /* tuner type */

#define dprintk     if (debug) printk

#ifdef MODULE
MODULE_PARM(debug,"i");
MODULE_PARM(type,"i");
#endif

/* addresses to scan, found only between 0x60 and 0x6f (7-bit) */
/*
unsigned short normal_i2c[] = {I2C_CLIENT_END};
unsigned short normal_i2c_range[] = {0x60,0x6f,I2C_CLIENT_END};
*/
unsigned short normal_i2c[] = {0x60, I2C_CLIENT_END};
unsigned short normal_i2c_range[] = {I2C_CLIENT_END};


/* magic definition of all other variables and things */
I2C_CLIENT_INSMOD;

/* unique ID allocation */
int tuner_id = 0;

struct i2c_driver driver;

struct tuner {
	int type;            /* chip type */
	int freq;            /* keep track of the current settings */
	int radio;
	int sysctl_id;
};

struct tunertype {
  char *name;
  unchar Vendor;
  unchar Type;
  
  ushort thresh1; /* frequency Range for UHF,VHF-L, VHF_H */   
  ushort thresh2;  
  unchar VHF_L;
  unchar VHF_H;
  unchar UHF;
  unchar config; 
  unchar I2C;
  ushort IFPCoff;
};

/*
 *	The floats in the tuner struct are computed at compile time
 *	by gcc and cast back to integers. Thus we don't violate the
 *	"no float in kernel" rule.
 */
struct tunertype tuners[] = {
        {"Temic PAL", TEMIC, PAL,
                16*140.25,16*463.25,0x02,0x04,0x01,0x8e,0xc2,623},
	{"Philips PAL_I", Philips, PAL_I,
	        16*140.25,16*463.25,0xa0,0x90,0x30,0x8e,0xc0,623},
	{"Philips NTSC", Philips, NTSC,
	        16*157.25,16*451.25,0xA0,0x90,0x30,0x8e,0xc0,732},
	{"Philips SECAM", Philips, SECAM,
	        16*168.25,16*447.25,0xA7,0x97,0x37,0x8e,0xc0,623},
	{"NoTuner", NoTuner, NOTUNER,
	         0        ,0        ,0x00,0x00,0x00,0x00,0x00,000},
	{"Philips PAL", Philips, PAL,
	        16*168.25,16*447.25,0xA0,0x90,0x30,0x8e,0xc0,623},
	{"Temic NTSC", TEMIC, NTSC,
	        16*157.25,16*463.25,0x02,0x04,0x01,0x8e,0xc2,732},
	{"TEMIC PAL_I", TEMIC, PAL_I,
	        16*170.00,16*450.00,0xa0,0x90,0x30,0x8e,0xc2,623},
};

#define TUNER_POR       0x80
#define TUNER_FL        0x40
#define TUNER_AFC       0x07

int tuner_getstatus(struct i2c_client *client)
{
	int byte = 0;
              
	if(-1 == (byte = i2c_smbus_read_byte(client))) {
		dprintk("tuner.o: tuner_getstatus i/o error.\n");
	}

	return byte;
}

int tuner_islocked (struct i2c_client *client)
{
        return (tuner_getstatus (client) & TUNER_FL);
}

int tuner_afcstatus (struct i2c_client *client)
{
        return (tuner_getstatus (client) & TUNER_AFC);
}
	
void set_tv_freq(struct i2c_client *client, int freq)
{
	struct tuner     *t   = (struct tuner*)client->data;
	struct tunertype *tun;
	unsigned char buffer[4];
	int rc;
	u8 config;
	u16 div;

	if (t->type == -1) {
                printk("tuner.o: tuner type not set\n");
		return;
	}
	tun = &tuners[t->type];
	if (freq < tun->thresh1) 
		config = tun->VHF_L;
	else if (freq < tun->thresh2) 
		config = tun->VHF_H;
	else
		config = tun->UHF;

        div=freq + tun->IFPCoff;
  	div&=0x7fff;

	buffer[0] = (div>>8) & 0x7f;
	buffer[1] = div      & 0xff;
	buffer[2] = tun->config;
	buffer[3] = config;

	if (4 != (rc = i2c_master_send(client,buffer,4)))
		printk("tuner.o: i2c i/o error: rc == %d (should be 4)\n",rc);
}

void set_radio_freq(struct i2c_client *client, int freq)
{
	struct tuner     *t   = (struct tuner*)client->data;
	struct tunertype *tun;
	unsigned char buffer[4];
	int rc;
	u8 config;
	u16 div;

	if (t->type == -1) {
                printk("tuner.o: tuner type not set\n");
		return;
	}
	tun = &tuners[t->type];
	config = 0xa5;
	div=freq + (int)(16*10.7);
  	div&=0x7fff;

	buffer[0] = (div>>8) & 0x7f;
	buffer[1] = div      & 0xff;
	buffer[2] = tun->config;
	buffer[3] = config;
	if (4 != (rc = i2c_master_send(client,buffer,4)))
		printk("tuner.o: i2c i/o error: rc == %d (should be 4)\n",rc);
}

#ifdef axa2
void tuner_sysctl(struct i2c_client *client, int operation, int ctl_name, int *nrels_mag, long *results)
{
	struct tuner *data = client->data;
	
	if (operation == SENSORS_PROC_REAL_INFO)
		*nrels_mag = 0;
	else if (operation == SENSORS_PROC_REAL_READ) {
		results[0] = data->freq;
		*nrels_mag = 1;
	} else if (operation == SENSORS_PROC_REAL_WRITE) {
		if (*nrels_mag == 1) {
			data->freq = results[0];
			set_tv_freq(client,data->freq);
		}
	}
}

/* these files are created for each detected tuner */
ctl_table tuner_dir_table_template[] = {
	{TUNER_SYSCTL_FREQ, "freq", NULL, 0, 0644, NULL, &sensors_proc_real,
	 &sensors_sysctl_real, NULL, &tuner_sysctl},
	{0}
};
#endif


/* this function is called by i2c_probe */
int tuner_detect(struct i2c_adapter *adapter, int address, unsigned short flags, int kind)
{
	struct i2c_client *client;
	struct tuner      *tuner;
	int err = 0;

	/* let's see whether this adapter can support what we need */
	if ( 0 == i2c_check_functionality(adapter, I2C_FUNC_SMBUS_WRITE_BYTE|I2C_FUNC_SMBUS_READ_BYTE)) {
		return 0;
	}

	/* allocate memory for client structure */
	client = kmalloc(sizeof(struct i2c_client), GFP_KERNEL);
        if (0 == client) {
		return -ENOMEM;
	}

	/* fill client structure */
	sprintf(client->name,"tuner (0x%02x)", address);
	client->id = tuner_id++;
	client->flags = 0;
	client->addr = address;
	client->adapter = adapter;
	client->driver = &driver;

	tuner = kmalloc(sizeof(struct tuner),GFP_KERNEL);
	if (NULL == tuner) {
		kfree(client);
                return -ENOMEM;
	}
	client->data = tuner;
	memset(tuner,0,sizeof(struct tuner));
	tuner->type = type;

	/* tell the i2c layer a new client has arrived */
	if (0 != (err = i2c_attach_client(client))) {
		kfree(client);
		return err;
	}

	/* register a new directory entry */
/*	if ((err = sensors_register_entry(client, "tuner", tuner_dir_table_template, THIS_MODULE)) < 0) {
		kfree(tuner);
		kfree(client);
		return err;
	}
	tuner->sysctl_id = err;
*/
	printk("tuner.o: detected @ 0x%02x on adapter %s\n",2*address,&client->adapter->name[0]);

	return 0;
}

int tuner_attach(struct i2c_adapter *adapter)
{
	return i2c_probe(adapter,&addr_data,&tuner_detect);
}

int tuner_detach(struct i2c_client *client)
{
	struct tuner *tuner = (struct tuner*)client->data;
	int err = 0;

//	sensors_deregister_entry(((struct tuner*)(client->data))->sysctl_id);

	if ( 0 != (err = i2c_detach_client(client))) {
		printk("tuner.o: Client deregistration failed, client not detached.\n");
		return err;
	}
	
	kfree(tuner);
	kfree(client);

	return 0;
}

int
tuner_command(struct i2c_client *client, unsigned int cmd, void *arg)
{
	struct tuner	*t    = (struct tuner*)client->data;
	int		*iarg = (int*)arg;
	
	switch (cmd) {
	case TUNER_SET_TYPE:
		if (t->type != -1)
			return 0;
		t->type = *iarg;
		dprintk("tuner.o: type set to %d (%s)\n",
			t->type,tuners[t->type].name);
		break;
		
	case TUNER_SET_TVFREQ:
		dprintk("tuner.o: tv freq set to %d.%02d (%u)\n",
			(*iarg)/16,(*iarg)%16*100/16, *iarg);
		set_tv_freq(client,*iarg);
		t->radio = 0;
		t->freq = *iarg;
		break;
		
	case TUNER_SET_RADIOFREQ:
		dprintk("tuner.o: radio freq set to %d.%02d\n",
			(*iarg)/16,(*iarg)%16*100/16);
		set_radio_freq(client,*iarg);
		t->radio = 1;
		t->freq = *iarg;
		break;
		
	case TUNER_IS_LOCKED: {
		int i = tuner_islocked(client);
		dprintk("tuner.o: phase is %slocked.\n",( i == 0 ? "not " : ""));
		return i;
		}
		
	case TUNER_GET_AFC_STATUS: {
		int i = tuner_afcstatus(client);
		dprintk("tuner.o: afcstatus is 0x%01x\n", i);
		return i;
		}
		
	default:
		return -EINVAL;
	}
	return 0;
}

void tuner_inc_use(struct i2c_client *client)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

void tuner_dec_use(struct i2c_client *client)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

struct i2c_driver driver = {
	"tuner driver",
	I2C_DRIVERID_TUNER,
	I2C_DF_NOTIFY,
	tuner_attach,
	tuner_detach,
	tuner_command,
	tuner_inc_use,
	tuner_dec_use
};

EXPORT_NO_SYMBOLS;

#ifdef MODULE
static
#else
extern
#endif
	int __init tuner_init(void)
{
	int res = 0;

	printk("tuner.o version %s (%s)\n",VERSION,DATE);

	if( 0 != (res = i2c_add_driver(&driver))) {
		printk("tuner.o: Driver registration failed, module not inserted.\n");
		return res;
	}
	
	return 0;
}

int __init tuner_cleanup(void)
{	
	int res = 0;

	if ( 0 != (res = i2c_del_driver(&driver))) {
		printk("tuner.o: Driver deregistration failed, module not removed.\n");
		return res;
	}
	
	return 0;
}

#ifdef MODULE

MODULE_AUTHOR("Markus Schroeder/Ralph Metzler/Gerd Knorr/Michael Hunold");
MODULE_DESCRIPTION("tuner driver");

int init_module(void)
{
	return tuner_init();
}

void cleanup_module(void)
{
	tuner_cleanup();
}

#endif

#ifdef MODULE
MODULE_LICENSE("GPL");
#endif
