#define DATE "20000701"
#define VERSION "1.2"

#include <linux/module.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/i2c.h>

#include "saa7111a.h"
#include "saa711x.c"

/* addresses to scan, found only at 0x24 or 0x25 */
static unsigned short normal_i2c[] = {I2C_SAA7111A, I2C_SAA7111A_2, I2C_CLIENT_END};
static unsigned short normal_i2c_range[] = {I2C_CLIENT_END};
/* magic definition of all other variables and things */
I2C_CLIENT_INSMOD;

static int saa7111a_command(struct i2c_client *client, unsigned int cmd, void *arg)
{
	int result = 0;

	dprintk("saa7111a.o: ==> saa7111a_command.\n");

	if( 0 == (result = saa711x_command(client, cmd, arg))) {
		return result;
	} 
	
	switch (cmd)
	{
		default: {
			return -EINVAL;
		}
	}

	return 0;
}

/* this function initalizes any saa7111a to useful values. please note:
   the values you see here are for my "multimedia extension board" --
   you might have to call 'saa7111a_command' via the
   'i2c_send_command'-call to set the values that work with your board */
static int saa7111a_initialize(struct i2c_client* client)
{
	int 	i = 0, result = 0;

	int	data[0x20] = {	  -1,   -1, 0xd8, 0x23, 0x00, 0x00, 0xf0, 0x30,
				0xa8, 0x02, 0x80, 0x47, 0x40, 0x00, 0x01,   -1,
				0xd0, 0x8c, 0x80, 0x30,   -1, 0x15, 0x04, 0x00,
				  -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1 };
				   
	dprintk("saa7111a.o: ==> saa7111a_initialize\n");

	for(i = 0; i < 0x20; i++) {
		if( -1 == data[i] ) {
			continue;
		}
		if ( 0 != (result = i2c_smbus_write_byte_data(client, i, data[i]))) {
			dprintk(KERN_ERR "saa7111a.o: could not initalize saa7111a #1 (%d), result: %d\n",i,result);
 			return -EFAULT;
		}
	}
/*
	printk("0x%08x\n",i2c_smbus_read_byte_data(client, 0x15));
	printk("0x%08x\n",i2c_smbus_read_byte_data(client, 0x16));
*/
	/* check for a valid mode */
	if( 0 > mode || mode > 2 ) {
		printk(KERN_ERR "saa7111a.o: invalid mode setting. using PAL as default\n");
		mode = 0;
	}

	/* use the existing saa7111a_command ioctls */
	i = mode;
	if ( 0 != saa7111a_command(client, SAA711X_SET_MODE, &i)) {
		dprintk(KERN_ERR "saa7111a.o: saa7111a_init error #2\n");
		return -EFAULT;
	}
	
	i  = SAA711X_USE_ANTIALIAS;
	i |= SAA711X_USE_AMPLIFIER; 
	if ( 0 != saa7111a_command(client, SAA711X_SET_FUSE, &i)) {
		dprintk(KERN_ERR "saa7111a.o: saa7111a_init error #3\n");
		return -EFAULT;
	}
		
	switch(mode) {
		case 1:
			i = 0;
			break;
		default:
			i = 3;
			break;
		}
	
	if ( 0 != saa7111a_command(client, SAA711X_SET_UPDATE_GAIN, &i)) {
		dprintk(KERN_ERR "saa7111a.o: saa7111a_init error #4\n");
		return -EFAULT;
	}

	switch(mode) {
		case 2:
			i = SAA711X_VIDEO_MODE_SECAM;
			break;
		case 1:
			i = SAA711X_VIDEO_MODE_NTSC;
			break;
		default:
			i = SAA711X_VIDEO_MODE_PAL;
			break;
		}
		
	if ( 0 != saa7111a_command(client, SAA711X_SET_NORM, &i)) {
		dprintk(KERN_ERR "saa7111a.o: saa7111a_init error #5\n");
		return -EFAULT;
	}

	return 0;
}

/* this function is called by i2c_probe */
int saa7111a_detect(struct i2c_adapter *adapter, int address, unsigned short flags, int kind)
{
	int result = 0;
	struct i2c_client* client = NULL;

	dprintk("saa7111a.o: ==> saa7111a_detect\n");
	
	if( 0 != (result = saa711x_detect(adapter, address, flags, kind, "saa7111a\0",&client))) {
		return result;
	}

	if( 0 != saa7111a_initialize(client) ) {
		printk("saa7111a.o: could not initialize chipset. continuing anyway.\n");
	}

	return 0;
}

static int saa7111a_attach(struct i2c_adapter *adapter)
{
	dprintk("saa7111a.o: ==> saa7111a_attach\n");

	return i2c_probe(adapter,&addr_data,&saa7111a_detect);
}

static struct i2c_driver driver = {
	"saa7111a driver",
	I2C_DRIVERID_SAA7111A,
	I2C_DF_NOTIFY,
	saa7111a_attach,
	saa711x_detach,
	saa7111a_command,
	saa711x_inc_use,
	saa711x_dec_use
};

EXPORT_NO_SYMBOLS;

#ifdef MODULE
MODULE_AUTHOR("Michael Hunold <michael@mihu.de>");
MODULE_DESCRIPTION("saa7111a driver");

int init_module(void)
{
	dprintk("saa7111a.o: ==> init_module\n");
	return saa711x_init();
}

void cleanup_module(void)
{
	dprintk("saa7111a.o: ==> cleanup_module\n");
	saa711x_cleanup();
}

#endif

#ifdef MODULE
MODULE_LICENSE("GPL");
#endif
