#include <linux/module.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/i2c.h>

#define DATE "200013010"
#define VERSION "0.1"

#ifdef MODULE
MODULE_PARM(debug,"i");
#endif

int debug = 0;	/* insmod parameter */
#define dprintk	if (debug) printk

/* addresses to scan */
unsigned short normal_i2c[] = {0x24, I2C_CLIENT_END};
unsigned short normal_i2c_range[] = {I2C_CLIENT_END};

/* magic definition of all other variables and things */
I2C_CLIENT_INSMOD;

/* unique ID allocation */
int saa7114_id = 0;

struct i2c_driver driver;

int saa7114_command(struct i2c_client *client, unsigned int cmd, void* arg)
{
	switch (cmd) {
		default:
			return -ENOIOCTLCMD;
	}

	return 0;
}

#define NUM_REGISTERS 0x20
int registers_dump[NUM_REGISTERS];

int registers_init[NUM_REGISTERS] = { 0x11, -1, 0xd8, 0x23, 0x00, 0x00, 0xf0, 0x30, 
0xa8, 0x02, 0x80, 0x47, 0x40, 0x00, 0x01, 0x00, 0xd0, 0x8c, 0x80, 0x30, 0x00, 0x15,
0x04, 0x00, 0x00, 0x00, 0x03, 0x04, 0x80, 0x00, 0x00, 0x81 };

int saa7114_detect(struct i2c_adapter *adapter, int address, unsigned short flags, int kind)
{
	struct	i2c_client *client;
	int result = 0;

	int i = 0;
			
	/* let's see whether this adapter can support what we need */
	if ( 0 == i2c_check_functionality(adapter, I2C_FUNC_SMBUS_READ_BYTE_DATA|I2C_FUNC_SMBUS_WRITE_BYTE_DATA)) {
		return 0;
	}

	/* allocate memory for client structure */
	client = kmalloc(sizeof(struct i2c_client), GFP_KERNEL);
        if (0 == client) {
		printk("saa7114.o: not enough kernel memory.\n");
		return -ENOMEM;
	}
	
	/* fill client structure */
	sprintf(client->name,"saa7114 (0x%02x)", address);
	client->id = saa7114_id++;
	client->flags = 0;
	client->addr = address;
	client->adapter = adapter;
	client->driver = &driver;
	client->data = NULL;

/* ...dump stuff begin */

	/* dump all registers_dump */
	for(i = 0; i < NUM_REGISTERS; i++) {
		if ( -1 == (registers_dump[i] = i2c_smbus_read_byte_data(client, i))) {
			dprintk("saa7114.o: could not read from saa7114.\n");
			kfree(client);
			return -EFAULT;
		}
	}

	for(i = 0; i < NUM_REGISTERS; i++) {
		printk("0x%02x, ",registers_dump[i]);
	}
	printk("\n");
/* ...dump stuff end */

	/* tell the i2c layer a new client has arrived */
	if (0 != (result = i2c_attach_client(client))) {
		kfree(client);
		return result;
	}

	printk("saa7114.o: detected @ 0x%02x on adapter %s\n",2*address,&client->adapter->name[0]);

/* ...init stuff begin */
/*
	for(i = 0; i < NUM_REGISTERS; i++) {
		if( -1 == registers_init[i] ) {
			continue;
		}
		if ( 0 != (result = i2c_smbus_write_byte_data(client, i, registers_init[i]))) {
			dprintk(KERN_ERR "saa7111a.o: could not initalize saa714 #1 (%d), result: %d\n",i,result);
 			return -EFAULT;
		}
	}
*/
/* ...init stuff begin */

	return 0;
}

int saa7114_attach(struct i2c_adapter *adapter)
{
	return i2c_probe(adapter,&addr_data,&saa7114_detect);
}

int saa7114_detach(struct i2c_client *client)
{
	int err = 0;

	if ( 0 != (err = i2c_detach_client(client))) {
		printk("saa7114.o: Client deregistration failed, client not detached.\n");
		return err;
	}
	
	kfree(client);

	return 0;
}

void saa7114_inc_use(struct i2c_client *client)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

void saa7114_dec_use(struct i2c_client *client)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

struct i2c_driver driver = {
	"saa7114 driver",
	I2C_DRIVERID_TDA9840,
	I2C_DF_NOTIFY,
	saa7114_attach,
	saa7114_detach,
	saa7114_command,
	saa7114_inc_use,
	saa7114_dec_use
};

EXPORT_NO_SYMBOLS;

#ifndef MODULE
extern
#endif
int __init saa7114_init(void)
{
	int res = 0;

	printk("saa7114.o: version %s (%s)\n",VERSION,DATE);

	if( 0 != (res = i2c_add_driver(&driver))) {
		printk("saa7114.o: Driver registration failed, module not inserted.\n");
		return res;
	}
	
	return 0;
}

int __init saa7114_cleanup(void)
{	
	int res = 0;

	if ( 0 != (res = i2c_del_driver(&driver))) {
		printk("saa7114.o: Driver deregistration failed, module not removed.\n");
		return res;
	}
	
	return 0;
}

#ifdef MODULE

MODULE_AUTHOR("[your-name-here] <[your-email-here]@blabla>");
MODULE_DESCRIPTION("saa7114 driver");

int init_module(void)
{
	return saa7114_init();
}

void cleanup_module(void)
{
	saa7114_cleanup();
}

#endif

#ifdef MODULE
MODULE_LICENSE("GPL");
#endif
