 /*
    saa711x.h - definitions for the generic i2c-driver for the both the
     
     - saa7111a (Enhanced Video Input Processor) and
     - saa7112 (saa7111a + high performace scaler)
     
    by Philips Semiconductors.
    
    Copyright (C) 1998,1999 Michael Hunold <michael@mihu.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __INCLUDED_SAA711X__
#define __INCLUDED_SAA711X__

#define	I2C_SAA7111A		0x24
#define	I2C_SAA7111A_2		0x25

#define	I2C_SAA7112		0x20

/* returns a short form of the status of the saa711x */
#define	SAA711X_GET_STATUS		_IOR('d', 1, int)
#define	SAA711X_STATUS_BAD	0	/* receiving inacceptable input */
#define	SAA711X_STATUS_GOOD	1	/* receiving acceptable input */

/* returns the whole status-byte of the saa711x
   - do what you want with it ;-) */
#define	SAA711X_GET_STATUS_BYTE		_IOR('d', 2, u8)

/* set the video norm */
#define	SAA711X_SET_NORM		_IOW('d', 3, int)
#define SAA711X_VIDEO_MODE_NTSC		1
#define SAA711X_VIDEO_MODE_PAL		2
#define SAA711X_VIDEO_MODE_SECAM	3

/* have a look at the saa7111a-specs from now on ;-) */

/* p.43: mode may range between 0 and 7 */
#define	SAA711X_SET_MODE		_IOW('d', 4, int)

/* p.44: the values may be or-ed (and set both) */
#define	SAA711X_SET_FUSE		_IOW('d', 5, int)
#define SAA711X_USE_ANTIALIAS		0x1
#define SAA711X_USE_AMPLIFIER		0x2

/* p.43: the update hysteresis for 9-bit-gain may
   range between 0 (=off) and 7 */
#define	SAA711X_SET_UPDATE_GAIN		_IOW('d', 6, int)
#define	SAA711X_SET_OUTPUT		_IOW('d', 7, int)
#define	SAA711X_ENABLE_OUTPUT		_IOW('d', 8, int)

#define	SAA711X_VBI_BYPASS		_IOW('d', 9, int)

/* direct access to subaddresses for other modules */
#include <asm/io.h>             /* definition of u8 */
struct saa711x_subaddress_access {
	u8 subaddress;
	u8 byte;
};
#define	SAA711X_SUBADDRESS_WRITE	_IOW('d', 9, struct saa711x_subaddress_access)

/* All unused bytes are reserverd. */
#define SAA711X_CHIP_VERSION		0x00
#define SAA711X_ANALOG_INPUT_CONTROL_1	0x02
#define SAA711X_ANALOG_INPUT_CONTROL_2	0x03
#define SAA711X_ANALOG_INPUT_CONTROL_3	0x04
#define SAA711X_ANALOG_INPUT_CONTROL_4	0x05
#define SAA711X_HORIZONTAL_SYNC_START	0x06
#define SAA711X_HORIZONTAL_SYNC_STOP	0x07
#define SAA711X_SYNC_CONTROL		0x08
#define SAA711X_LUMINANCE_CONTROL	0x09
#define SAA711X_LUMINANCE_BRIGHTNESS	0x0A
#define SAA711X_LUMINANCE_CONTRAST	0x0B
#define SAA711X_CHROMA_SATURATION	0x0C
#define SAA711X_CHROMA_HUE_CONTROL	0x0D
#define SAA711X_CHROMA_CONTROL		0x0E
#define SAA711X_FORMAT_DELAY_CONTROL	0x10
#define SAA711X_OUTPUT_CONTROL_1	0x11
#define SAA711X_OUTPUT_CONTROL_2	0x12
#define SAA711X_OUTPUT_CONTROL_3	0x13
#define SAA711X_V_GATE_1_START		0x15
#define SAA711X_V_GATE_1_STOP		0x16
#define SAA711X_V_GATE_1_MSB		0x17
#define SAA711X_TEXT_SLICER_STATUS	0x1A
#define SAA711X_DECODED_BYTES_OF_TS_1	0x1B
#define SAA711X_DECODED_BYTES_OF_TS_2	0x1C
#define SAA711X_STATUS_BYTE		0x1F

#endif
