#include <linux/module.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/i2c.h>

#define DATE "20001410"
#define VERSION "1.0"

#ifdef MODULE
MODULE_PARM(debug,"i");
#endif

int debug = 0;	/* insmod parameter */
#define dprintk	if (debug) printk

/* addresses to scan, found only at 0x42 (7-Bit) */
unsigned short normal_i2c[] = {I2C_CLIENT_END};
unsigned short normal_i2c_range[] = {0x00, 0x74, I2C_CLIENT_END};

/* magic definition of all other variables and things */
I2C_CLIENT_INSMOD;

/* unique ID allocation */
int detect_id = 0;

struct i2c_driver driver;

int detect_command(struct i2c_client *client, unsigned int cmd, void* arg)
{
	switch (cmd) {
		default:
			return -ENOIOCTLCMD;
	}

	return 0;
}

int detect_detect(struct i2c_adapter *adapter, int address, unsigned short flags, int kind)
{
	printk("i2c-detect.o: detected i2c-client @ 0x%02x on adapter %s\n",2*address,&adapter->name[0]);
	return 0;
}

int detect_attach(struct i2c_adapter *adapter)
{
	return i2c_probe(adapter,&addr_data,&detect_detect);
}

int detect_detach(struct i2c_client *client)
{
	return 0;
}

void detect_inc_use(struct i2c_client *client)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

void detect_dec_use(struct i2c_client *client)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

struct i2c_driver driver = {
	"detect driver",
	I2C_DRIVERID_EXP0,
	I2C_DF_NOTIFY,
	detect_attach,
	detect_detach,
	detect_command,
	detect_inc_use,
	detect_dec_use
};

EXPORT_NO_SYMBOLS;

#ifndef MODULE
extern
#endif
int __init detect_init(void)
{
	int res = 0;

	printk("i2c-detect.o version %s (%s)\n",VERSION,DATE);

	if( 0 != (res = i2c_add_driver(&driver))) {
		printk("i2c-detect.o: Driver registration failed, module not inserted.\n");
		return res;
	}
	
	return 0;
}

int __init detect_cleanup(void)
{	
	int res = 0;

	if ( 0 != (res = i2c_del_driver(&driver))) {
		printk("i2c-detect.o: Driver deregistration failed, module not removed.\n");
		return res;
	}
	
	return 0;
}

#ifdef MODULE

MODULE_AUTHOR("Michael Hunold <michael@mihu.de>");
MODULE_DESCRIPTION("detect driver");

int init_module(void)
{
	return detect_init();
}

void cleanup_module(void)
{
	detect_cleanup();
}

#endif
