/* search and replace for "I2C_DUMMY" and "i2c_dummy" */

#include <linux/module.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/i2c.h>

#define I2C_I2C_DUMMY             0x42

#define DATE "20001031"
#define VERSION "0.1"

#ifdef MODULE
MODULE_PARM(debug,"i");
#endif

int debug = 0;
#define dprintk	if (debug) printk

/* addresses to scan */
unsigned short normal_i2c[] = {I2C_I2C_DUMMY, I2C_CLIENT_END};
unsigned short normal_i2c_range[] = {I2C_CLIENT_END};

/* magic definition of all other variables and things */
I2C_CLIENT_INSMOD;

/* unique ID allocation */
int i2c_dummy_id = 0;

struct i2c_driver driver;

int i2c_dummy_command(struct i2c_client *client, unsigned int cmd, void* arg)
{
/*
	int result = 0;
*/
	switch (cmd) {
		default:
			return -ENOIOCTLCMD;
	}

	return 0;
}

int i2c_dummy_detect(struct i2c_adapter *adapter, int address, unsigned short flags, int kind)
{
	struct	i2c_client *client;
	int result = 0;

	/* let's see whether this adapter can support what we need */
	if ( 0 == i2c_check_functionality(adapter, I2C_FUNC_SMBUS_READ_BYTE_DATA|I2C_FUNC_SMBUS_WRITE_BYTE_DATA)) {
		return 0;
	}

	/* allocate memory for client structure */
	client = kmalloc(sizeof(struct i2c_client), GFP_KERNEL);
        if (0 == client) {
		printk("i2c_dummy.o: not enough kernel memory.\n");
		return -ENOMEM;
	}
	
	/* fill client structure */
	sprintf(client->name,"i2c_dummy (0x%02x)", address);
	client->id = i2c_dummy_id++;
	client->flags = 0;
	client->addr = address;
	client->adapter = adapter;
	client->driver = &driver;
	client->data = NULL;

	/* tell the i2c layer a new client has arrived */
	if (0 != (result = i2c_attach_client(client))) {
		kfree(client);
		return result;
	}
	
	printk("i2c_dummy.o: detected @ 0x%02x on adapter %s\n",2*address,&client->adapter->name[0]);

	return 0;
}

int i2c_dummy_attach(struct i2c_adapter *adapter)
{
	return i2c_probe(adapter,&addr_data,&i2c_dummy_detect);
}

int i2c_dummy_detach(struct i2c_client *client)
{
	int err = 0;

	if ( 0 != (err = i2c_detach_client(client))) {
		printk("i2c_dummy.o: Client deregistration failed, client not detached.\n");
		return err;
	}
	
	kfree(client);

	return 0;
}

void i2c_dummy_inc_use(struct i2c_client *client)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

void i2c_dummy_dec_use(struct i2c_client *client)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

struct i2c_driver driver = {
	"i2c_dummy driver",
	I2C_DRIVERID_EXP0,
	I2C_DF_NOTIFY,
	i2c_dummy_attach,
	i2c_dummy_detach,
	i2c_dummy_command,
	i2c_dummy_inc_use,
	i2c_dummy_dec_use
};

EXPORT_NO_SYMBOLS;

#ifndef MODULE
extern
#endif
int __init i2c_dummy_init(void)
{
	int res = 0;

	printk("i2c_dummy.o version %s (%s)\n",VERSION,DATE);

	if( 0 != (res = i2c_add_driver(&driver))) {
		printk("i2c_dummy.o: Driver registration failed, module not inserted.\n");
		return res;
	}
	
	return 0;
}

int __init i2c_dummy_cleanup(void)
{	
	int res = 0;

	if ( 0 != (res = i2c_del_driver(&driver))) {
		printk("i2c_dummy.o: Driver deregistration failed, module not removed.\n");
		return res;
	}
	
	return 0;
}

#ifdef MODULE

MODULE_AUTHOR("[your-name-here] <[your-email-address-here]>");
MODULE_DESCRIPTION("i2c_dummy driver");

int init_module(void)
{
	return i2c_dummy_init();
}

void cleanup_module(void)
{
	i2c_dummy_cleanup();
}

#endif
