#define DATE "20000701"
#define VERSION "1.1"

#include <linux/module.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/videotext.h>
#include <linux/i2c.h>

#include <linux/poll.h>

#include "videodevX/videodev.h"

#define	I2C_SAA5281 0x11

#define NUM_DAUS	1

#define MAX_VTX_DEVICES 4
int unit_vtx[MAX_VTX_DEVICES] = { 192, 193, 194, 195, };

#ifdef MODULE
MODULE_PARM(debug,"i");
MODULE_PARM_DESC(debug, "debug level - 0: off; 1: normal; 2: verbose");

MODULE_PARM(unit_vtx, "1-"__MODULE_STRING(MAX_VTX_DEVICES)"i");
MODULE_PARM_DESC(unit_vtx, "minor device-node(s) to be used (for vtx)");
#endif

int debug = 0;	/* insmod parameter */
#define dprintk	if (debug) printk

/* addresses to scan, found only at 0x11 (7-Bit) */
unsigned short normal_i2c[] = {I2C_SAA5281, I2C_CLIENT_END};
unsigned short normal_i2c_range[] = {I2C_CLIENT_END};

/* magic definition of all other variables and things */
I2C_CLIENT_INSMOD;

/* unique ID allocation */
int saa5281_id = 0;

struct i2c_driver driver;

/*********************/
/* video4linux-stuff */
/*********************/

#define SAA5281_ADVANCEDCONTROL		0
#define SAA5281_MODE			1
#define SAA5281_PAGEREQUESTADR		2
#define SAA5281_PAGEREQUESTDATA		3
#define SAA5281_DISPLAYCHAPTER		4
#define SAA5281_DISPLAYCONTRN		5
#define SAA5281_DISPLAYCONTREX		6
#define SAA5281_DISPLAYMODE		7
#define SAA5281_ACTIVECHAPTER		8
#define SAA5281_CURSORROW		9
#define SAA5281_CURSORCOL		10
#define SAA5281_CURSORDATA		11
#define SAA5281_STATUS			11
#define SAA5281_ADVCONTROLA		12
#define SAA5281_ADVCONTROLB		13 

int saa5281_open(struct v4l2_device *v, int flags, void **idptr)
{
	struct i2c_client *client = v->priv;
	int result = 0;

	struct	i2c_msg m;
	unsigned char	c[4];

	/* subadress-access is required so we must use i2c-messages */
	memset(&m,0x0,sizeof(struct i2c_msg));
	m.addr = client->addr;
	m.buf = &c[0];

	c[0] = SAA5281_ADVANCEDCONTROL;
	c[1] = 0;
	c[2] = 0x50; /* 5281_MODE + 8-BIT*/
	c[3] = 0x80; /* 5281_PAGEREQUESTADR */
	m.len = 4;

	if ( 1 != (result = i2c_transfer(client->adapter,&m,1))) {
		dprintk(KERN_ERR "saa5281: could not initalize saa5281 #1, result:%d\n",result);
 		return -EFAULT;
	}

	c[0] = SAA5281_ADVCONTROLB;
	c[1] = 0x98; /*pointers enable,VPS + enhance mode */
	m.len = 2;

	if ( 1 != (result = i2c_transfer(client->adapter,&m,1))) {
		dprintk(KERN_ERR "saa5281: could not initalize saa5281 #2, result:%d\n",result);
 		return -EFAULT;
	}

	c[0] = SAA5281_ADVCONTROLA;
	c[1] = 0x00; /*set Hamming 8/4 to off */
	m.len = 2;

	if ( 1 != (result = i2c_transfer(client->adapter,&m,1))) {
		dprintk(KERN_ERR "saa5281: could not initalize saa5281 #2, result:%d\n",result);
 		return -EFAULT;
	}

	MOD_INC_USE_COUNT;
	*idptr = v;

	return 0;
}

void saa5281_close(void *id)
{
/*
	struct v4l2_device *v = id;
*/	
	MOD_DEC_USE_COUNT;
}

int i2c_master_recv_subadress(struct i2c_client *client,const char *buf ,int count, int subadress)
{
	int	ret;
	struct	i2c_msg msg[2];
	
	msg[0].addr	= client->addr;
	msg[0].flags	= client->flags & I2C_M_TEN;
	msg[0].len	= 1;
	(const char*)msg[0].buf = (const char*)&subadress;

	msg[1].addr	= client->addr;
	msg[1].flags	= client->flags & I2C_M_TEN;
	msg[1].flags    |= I2C_M_RD;
	msg[1].len	= count;
	(const char*)msg[1].buf = (const char*)buf;
	
	ret = i2c_transfer(client->adapter, &msg[0], 2);

	/* if everything went ok (i.e. 2 msg transmitted),
	   return #bytes transmitted, else return error code.
	 */
	return (ret == 2 ) ? count : ret;
}

int saa5281_ioctl(void *id, unsigned int cmd, void *arg) 
{
	struct v4l2_device *v = id;
	struct i2c_client *client = v->priv;

	switch(cmd) 
	{
		case VTXIOCGETINFO: 
		{
			vtx_info_t info;

			dprintk(KERN_ERR "saa5281: VTXIOCGETINFO called.\n");				
			
			/* dummy stuff */
			info.version_major = 1;
			info.version_minor = 6;
			info.numpages = 1;
			info.cct_type = SAA5248;
			
			if(copy_to_user((void*)arg, &info, sizeof(vtx_info_t)))
				return -EFAULT;
				
			return 0;
		}

		case VTXIOCCLRPAGE: 
		{
			vtx_pagereq_t req;
			dprintk(KERN_ERR "saa5281: VTXIOCCLRPAGE called.\n");				
			return 0;
		}

		case VTXIOCCLRFOUND: 
		{
			vtx_pagereq_t req;
      
			dprintk(KERN_ERR "saa5281: VTXIOCCLRFOUND called.\n");				
			return 0;
		}

		case VTXIOCPAGEREQ: 
		{
			vtx_pagereq_t req;
			u32 magazine, tens, units;

			int chapter = 0, result = 0;

			struct	i2c_msg m;
			unsigned char	c[6];
			
			if(copy_from_user(&req, (void*)arg, sizeof(vtx_pagereq_t)))
				return -EFAULT;
			
			magazine =  (u32)req.page/0x100;
			tens	 = ((u32)req.page-(magazine*0x100))/0x10;
			units	 =  (u32)req.page-(magazine*0x100)-(tens*0x10);
			
			dprintk(KERN_ERR "saa5281: VTXIOCPAGEREQ called, p:0x%03x,m:0x%01x,t:0x%01x,u:0x%01x\n",req.page,magazine,tens,units);				

			/* subadress-access is required so we must use i2c-messages */
			memset(&m,0x0,sizeof(struct i2c_msg));
			m.addr = client->addr;
			m.buf = &c[0];

			c[0] = SAA5281_PAGEREQUESTADR;
			c[1] = (chapter << 4) | 0x87;
			c[2] = chapter;
			c[3] = magazine | 0x08 | 0x10;
			c[4] = tens | 0x10;
			c[5] = units | 0x10;
			m.len = 6;
			
/*			for(result = 0; result < 6; result++){
				printk("%d:0x%02x\n",result,c[result]);
			}
*/			
			if ( 1 != (result = i2c_transfer(client->adapter,&m,1))) {
				dprintk(KERN_ERR "saa5281: VTXIOCPAGEREQ: could not write to saa5281, result:%d\n",result);
 				return -EFAULT;
			}
			dprintk(KERN_ERR "saa5281: VTXIOCPAGEREQ called, p:0x%03x,m:0x%01x,t:0x%01x,u:0x%01x\n",req.page,magazine,tens,units);				
	
			return 0;
		}

		case VTXIOCGETSTAT: 
		{
			vtx_pagereq_t req;
			vtx_pageinfo_t info;

			u8 infobits[10];

			int loop;	
	
			int chapter = 0, result = 0;
			u8 byte = 0;
			
			struct	i2c_msg m;
			unsigned char	c[6];


			if(copy_from_user(&req, (void*)arg, sizeof(vtx_pagereq_t)))
				return -EFAULT;

			dprintk(KERN_ERR "saa5281: VTXIOCGETSTAT called, p:0x%03x\n",req.page);				

			if (req.pgbuf < 0 || req.pgbuf >= NUM_DAUS)
				return -EINVAL;

			/* subadress-access is required so we must use i2c-messages */
			memset(&m,0x0,sizeof(struct i2c_msg));
			m.addr = client->addr;
			m.buf = &c[0];

			/* check the signal quality */
			
			/* prepare to read advanced control at register 11*/
			byte = 0x01;
			if ( 0 != (result = i2c_smbus_write_byte_data(client, SAA5281_ADVANCEDCONTROL, byte))) {
				dprintk(KERN_ERR "saa5281: VTXIOCPAGEREQ: could not read from saa5281, result:%d\n",result);
				return -EFAULT;
			}

			/* read status-byte */
			if ( -1 == (result = i2c_smbus_read_byte_data(client, SAA5281_STATUS))) {
				dprintk(KERN_ERR "saa5281: VTXIOCPAGEREQ: could not read from saa5281, result:%d\n",result);
				return -EFAULT;
			}
			byte = result;
			
			printk("saa5281: vcs-signal:%d, text-signal:%d\n",(byte&0x1),((byte&0x2)>>1));

			/* prepare to read data-byte at register 11*/
			byte = 0x00;
			if ( 0 != (result = i2c_smbus_write_byte_data(client, SAA5281_ADVANCEDCONTROL, byte))) {
				dprintk(KERN_ERR "saa5281: VTXIOCPAGEREQ: could not read from saa5281, result:%d\n",result);
				return -EFAULT;
			}

			/* select information to read */
			c[0] = SAA5281_ACTIVECHAPTER;
			c[1] = chapter;
			c[2] = 25;
			c[3] = 0;
			m.len = 4;

			if ( 1 != (result = i2c_transfer(client->adapter,&m,1))) {
				dprintk(KERN_ERR "saa5281: VTXIOCPAGEREQ: could not write to saa5281, result:%d\n",result);
 				return -EFAULT;
			}

			for(loop = 0; loop < 10; loop++) {
			
				/* read out information */
				if ( -1 == (result = i2c_smbus_read_byte_data(client, SAA5281_CURSORDATA))) {
					dprintk(KERN_ERR "saa5281: VTXIOCPAGEREQ: could not read from saa5281, result:%d\n",result);
					return -EFAULT;
				}
				infobits[loop] = result;
			if(loop ==0 || loop ==1 || loop ==8 || loop == 9)
				printk("%d infobits: 0x%02x\n",loop, infobits[loop]);

			}
				
		info.pagenum = ((infobits[8] << 8) & 0x700) | ((infobits[1] << 4) & 0xf0) | (infobits[0] & 0x0f);
		info.minute = ((infobits[3] << 4) & 0x70) | (infobits[2] & 0x0f);
		info.hour = ((infobits[5] << 4) & 0x30) | (infobits[4] & 0x0f);

		info.delete = !!(infobits[3] & 8);

		if (info.pagenum < 0x100)
			info.pagenum += 0x800;
			
		info.charset 	 = ((infobits[7] >> 1) & 7);
		info.headline	 = !!(infobits[5] & 4);
		info.subtitle 	 = !!(infobits[5] & 8);
		info.supp_header = !!(infobits[6] & 1);
		info.update 	 = !!(infobits[6] & 2);
		info.inter_seq 	 = !!(infobits[6] & 4);
		info.dis_disp 	 = !!(infobits[6] & 8);
		info.serial 	 = !!(infobits[7] & 1);
/*
		info.notfound 	 = !!(infobits[9] & 0x20);
*/

		info.notfound 	 = !!(infobits[8] & 0x10);
		info.pblf 	 = !!(infobits[9] & 0x20);
		info.hamming	 = 0;
		
		printk("infos: page:0x%03x, nf:%d, pblf:%d\n",info.pagenum, info.notfound, info.pblf);
		
		if(copy_to_user(req.buffer, &info, sizeof(vtx_pageinfo_t)))
			return -EFAULT;

		return 0;

		}
		case VTXIOCGETPAGE: 
		{
			vtx_pagereq_t req;
			int start, end;

			struct	i2c_msg m;
			unsigned char	c[6];
			
			int chapter = 0, result = 0;
			
			u8 	byte = 0;
			int	size = 0;	 
			u8	data[960];
			
			if(copy_from_user(&req, (void*)arg, sizeof(vtx_pagereq_t)))
				return -EFAULT;

			dprintk(KERN_ERR "saa5281: VTXIOCGETPAGE called, s:%d, e:%d\n",req.start,req.end);				

			if (req.pgbuf < 0 || req.pgbuf >= NUM_DAUS )
				return -EINVAL;

			if( req.start < 0 || req.end < 0 || req.start > 960 || req.end > 960)
				return 0;
				
			dprintk(KERN_ERR "saa5281: VTXIOCGETPAGE called, s:%d, e:%d\n",req.start,req.end);				
				
			/* prepare to read advanced control at register 11*/
			byte = 0x00;
			if ( 0 != (result = i2c_smbus_write_byte_data(client, SAA5281_ADVANCEDCONTROL, byte))) {
				dprintk(KERN_ERR "saa5281: VTXIOCGETPAGE: could not read from saa5281, result:%d\n",result);
				return -EFAULT;
			}

			/* subadress-access is required so we must use i2c-messages */
			memset(&m,0x0,sizeof(struct i2c_msg));
			m.addr = client->addr;
			m.buf = &c[0];

			c[0] = SAA5281_ACTIVECHAPTER;
			c[1] = chapter;
			c[2] = 0; /* req.start % 40;  row */
			c[3] = 0; /* req.start / 40;  column */
			m.len = 4;

			if ( 1 != (result = i2c_transfer(client->adapter,&m,1))) {
				dprintk(KERN_ERR "saa5281: VTXIOCGETPAGE: could not write to saa5281, result:%d\n",result);
 				return -EFAULT;
			}

			size = req.end - req.start + 1;

			/* read out information */
			if ( 960 /*size*/ != (result = i2c_master_recv_subadress(client,&data[0/*req.start*/],960/*size*/,SAA5281_CURSORDATA)) ) {
				dprintk(KERN_ERR "saa5281: VTXIOCGETPAGE: could not read from saa5281, result:%d\n",result);
				return -EFAULT;
			}

			if(copy_to_user(req.buffer, &data[req.start], req.end - req.start + 1))
				return -EFAULT;

			return 0;
		}

		case VTXIOCSTOPDAU: 
		{
			vtx_pagereq_t req;

			dprintk(KERN_ERR "saa5281: VTXIOCSTOPDAU called.\n");
			return 0;
		}

		case VTXIOCPUTPAGE: 
		case VTXIOCSETDISP: 
		case VTXIOCPUTSTAT: 
			return 0;
			
		case VTXIOCCLRCACHE: 
		{
			return 0;
		}

		case VTXIOCSETVIRT: 
		{
			return 0;
		}
	}

	return -EINVAL;
}

/*********************/
/* i2c-interface-stuff */
/*********************/

int
saa5281_command(struct i2c_client *client, unsigned int cmd, void* arg)
{
	return -EINVAL;
}

int saa5281_detect(struct i2c_adapter *adapter, int address, unsigned short flags, int kind)
{
	struct	i2c_client *client;
	struct	v4l2_device *vtx;

	u8	byte = 0x0;
	int 	result = 0;
			
	/* let's see whether this adapter can support what we need */
	if ( 0 == i2c_check_functionality(adapter, I2C_FUNC_SMBUS_READ_BYTE_DATA|I2C_FUNC_SMBUS_WRITE_BYTE_DATA)) {
		return 0;
	}

	/* allocate memory for client structure */
	client = kmalloc(sizeof(struct i2c_client), GFP_KERNEL);
        if (0 == client) {
		return -ENOMEM;
	}
	
	/* fill client structure */
	sprintf(client->name,"saa5281 (0x%02x)", address);
	client->id = saa5281_id;
	client->flags = 0;
	client->addr = address;
	client->adapter = adapter;
	client->driver = &driver;
	client->data = NULL;

	/* tell the i2c layer a new client has arrived */
	if (0 != (result = i2c_attach_client(client))) {
		kfree(client);
		return result;
	}

	/* now create a video4linux device */
	vtx = (struct v4l2_device *)kmalloc(sizeof(struct v4l2_device), GFP_KERNEL);
	if(vtx == NULL){
		return -ENOMEM;
	}
	memset(vtx, 0x0, sizeof(struct v4l2_device));
	
	sprintf(vtx->name, "%s (%d)", adapter->name, saa5281_id);
	vtx->type		= V4L2_TYPE_VTX;
	vtx->minor		= unit_vtx[saa5281_id]; /* fixme */
	vtx->open 		= saa5281_open;
	vtx->close 		= saa5281_close;
	vtx->read 		= NULL; /* saa5281_read; */
	vtx->write 		= NULL; /* saa5281_write; */
	vtx->ioctl 		= saa5281_ioctl;
	vtx->mmap		= NULL; /* saa5281_mmap; */
	vtx->poll		= NULL; /* saa5281_poll; */
	vtx->initialize 	= NULL; /* saa5281_init_done; */
	vtx->priv = client;		 

	saa5281_id++;

	client->data = vtx;

	if ( 0 != (result = v4l2_register_device(vtx))) {
		printk(KERN_INFO "saa5281.o: cannot register v4l2-device. aborting.\n");
		return result;
	}

	printk("saa5281.o: detected @ 0x%02x on adapter %s\n",2*address,&client->adapter->name[0]);

	return 0;
}

int saa5281_attach(struct i2c_adapter *adapter)
{
	return i2c_probe(adapter,&addr_data,&saa5281_detect);
}

int saa5281_detach(struct i2c_client *client)
{
	struct v4l2_device *vtx = client->data;
	int err = 0;

	if ( 0 != (err = i2c_detach_client(client))) {
		printk("saa5281.o: Client deregistration failed, client not detached.\n");
		return err;
	}

	v4l2_unregister_device(vtx);
	
	kfree(vtx);
	kfree(client);

	return 0;
}


void saa5281_inc_use(struct i2c_client *client)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

void saa5281_dec_use(struct i2c_client *client)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

struct i2c_driver driver = {
	"saa5281 driver",
	I2C_DRIVERID_SAA5281,
	I2C_DF_NOTIFY,
	saa5281_attach,
	saa5281_detach,
	saa5281_command,
	saa5281_inc_use,
	saa5281_dec_use
};

EXPORT_NO_SYMBOLS;

#ifdef MODULE
static
#else
extern
#endif
	int __init saa5281_init(void)
{
	int res = 0;

	printk("saa5281.o version %s (%s)\n",VERSION,DATE);

	if( 0 != (res = i2c_add_driver(&driver))) {
		printk("saa5281.o: Driver registration failed, module not inserted.\n");
		return res;
	}
	
	return 0;
}

int __init saa5281_cleanup(void)
{	
	int res = 0;

	if ( 0 != (res = i2c_del_driver(&driver))) {
		printk("saa5281.o: Driver deregistration failed, module not removed.\n");
		return res;
	}
	
	return 0;
}

#ifdef MODULE

MODULE_AUTHOR("Michael Hunold <michael@mihu.de>");
MODULE_DESCRIPTION("saa5281 driver");

int init_module(void)
{
	return saa5281_init();
}

void cleanup_module(void)
{
	saa5281_cleanup();
}

#endif
