// compile with while in utils directory: gcc -I../. bcs.c -o bcs

#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>

#include "videodevX/videodev.h"

int main(int argc, char** argv)
{
	struct v4l2_capability  caps;
	struct v4l2_window	window;

	struct v4l2_queryctrl	ctrl;
	int id = V4L2_CID_BASE;
	
	struct v4l2_control	control;
			
	// open the device in "non-capturing" mode,
	// ie. you can change values even if "xawtv" 
	// is running in parallel		
	int vid = open("/dev/video", O_NONCAP);
        if(vid < 0) {
                printf("No such video device\n");
                return -1;
        }

	// retrieve the capabilites of the device and
	// see if the fit our needs
        if( 0 != ioctl(vid, VIDIOC_QUERYCAP, &caps)) {
                return -1;
        }

	// loop through all available controls of the device
	// and print out the name of each control

	// the docs say that we can simply iterate through
	// VIDIOC_QUERYCTRL unless we get an -EINVAL back.
	// we simply increment the id from V4L2_CID_BASE up.
	while(1) {
		ctrl.id = id++;
		if( 0 != ioctl(vid, VIDIOC_QUERYCTRL, &ctrl)) {
			break;
        	}
		if( ctrl.flags == V4L2_CTRL_FLAG_DISABLED) {
			continue;
		}
		printf("found: %s\n",ctrl.name);	
	}

	// let's change the brightness control. It's a standard
	// control, so we can use the id V4L2_CID_BRIGHTNESS
	// Also possible V4L2_CID_CONTRAST, V4L2_CID_SATURATION, ...
	control.id = V4L2_CID_BRIGHTNESS;

	// get the old value for brigthness
	if( 0 != ioctl(vid, VIDIOC_G_CTRL, &control)) {
                return -1;
	}
	printf("old value: %d\n", control.value);

	// change the value and write it to the device
	control.value /= 2;
	printf("new value: %d\n", control.value);
	if( 0 != ioctl(vid, VIDIOC_S_CTRL, &control)) {
                return -1;
	}
		
	// clean up
	close(vid);
	return 0;	
}


