/*  iic_write.c
**  write a sequence of bytes to the iic bus 
**
** Author : H. Hackmann
** Date 18.02.2000  Hac
**      Initial Version
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/i2c.h>

#define MAX_MESSAGE 40                /* maximum message length */

char *prgname,                        /* needed for messages */
     *device;

char          default_device[] = "/dev/i2c-0";
unsigned char message[MAX_MESSAGE];
unsigned char slave;

/********************     Usage Message    ************************/

void usage(void) {
    fprintf(stderr, "usage : %s [options] <slave> <message>\n"
            "  writes <message> to the iic slave <slave>. \n" 
            "  <slave> is the iic slave address in 8bit notation, \n"
            "  <message> is a sequence of bytes each as a separate arg \n"
            "  options: -b n : use number base n instead of 16 (hex)\n"
            "           -d s : use device s instead of %s\n"
            "           -h   : print this message, then exit\n"
	    , prgname, default_device);
    exit(1);
}

/********************   fatal error, abort   ************************/

void fatal(char *msg) { 
    fprintf(stderr, "%s: error - %s, exiting\n", prgname, msg);
    exit(2);
}

/******************************************************************/

unsigned char get_byte(char *str, int base) {
    int byte;
    byte = strtol(str, NULL, base);
    if ((byte < 0) || (byte > 255))
        fatal( "argument out of range");
    return byte;
}

/******************************************************************/

int main(int argc, char *argv[]) {
    extern char *optarg;
    extern int  optind, optopt;
    char        c;
    int         base = 16;
    int         handle, length;

    prgname = argv[0];
    device = default_device;
 
    /* process command line options */

    while ((c = getopt(argc, argv, "b:d:h")) != -1) {
        switch(c) {
           case 'b' : base = strtol(optarg, NULL, 10);
                      if((base < 2) || (base > 35))
                          usage();
            	       break;
            case 'd' : device = optarg;	
                       break;	
            default  : usage();
        }
    }

    /* get the slave address */
    if(argv[optind] == NULL)
       usage();
    slave = get_byte(argv[optind++], base);

    /* all remaining args are message bytes */
    length = 0;
    while(argv[optind] != NULL) {
        if(length >= MAX_MESSAGE - 1 )
            fatal("message too long");
        message[length] = get_byte(argv[optind++], base);
        length++;
    }

    /* open the I2c interface */
    handle = open(device,O_RDWR);
    if(handle < 0 )
        fatal(strerror(errno));
	
    /* set the proper I2C slave */
    if (ioctl(handle, I2C_SLAVE_FORCE, slave >> 1) < 0)
        fatal(strerror(errno));
        
    /* finally write the message */
    if(write(handle, message, length) != length)
        fatal(strerror(errno));
	
    /* done, cleanup */
    close(handle);
    return(0);
}

