/* 
     mxb_tool - a program to adjust some hardware-dependend stuff
     on the 'Multimedia eXtension Board' by Siemens - Nixdorf.

    Copyright (C) 1998-2001 Michael Hunold <michael@mihu.de>
   
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/    

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <string.h>

#include <linux/types.h>
#include <linux/i2c.h>

#include "videodevX/videodev.h"
#include "extensions/mxb_v4l2.h"

#define DEVICE "/dev/video0"
char g_device[64];

void usage(const char *prog)
{
	char* tmp = NULL;
	int i = 0;
		
	if(NULL != (tmp = strrchr(prog,'/'))) {
		prog = tmp+1;
	}
	
	fprintf(stderr,
		"\n'%s' is a command-line tool to adjust some hardware\n"
		"dependend stuff of the 'Multimedia eXtension Board'.\n"
		"\n"
		"usage: %s [ options ]\n"
		"\n"
		"options:\n"
		"  -v 'device'		capture device                    [%s]\n"
		"  -m 'mode'		audio mode of the current channel\n"
		"  -l 'to-line-out'	input to switch to line-out\n"
		"  -c 'to-cdrom-out'	input to switch to cdrom out\n"
		"  -f 'freq'		set frequency of tuner\n"
	    	"  -s			silent, don't print anything\n"
	    	"  -h			this help\n"
		"\n",
		prog, prog, g_device);
	fprintf(stderr, "\nAvailable inputs:\n");
	for(i = 0; i < MXB_AUDIOS; i++) {
		fprintf(stderr,"%d:%s, ",i, mxb_audios[i].name);				
	}
	fprintf(stderr,"\b\b  \n\n");
	exit(1);
}


int main(int argc, char** argv)
{
	struct v4l2_capability	caps;
	int vid = 0;
	int mode = -1, line = -1, cdrom = -1, silent = 0; 
	float freq = -1;
	
	int c;

	strcpy(g_device, DEVICE);

	/* parse options */
	for(;;) {
		if (-1 == (c = getopt(argc, argv, "v:m:l:c:f:hs"))) {
			break;
		}
	switch (c) {
		case 's': {
			silent = 1;
			break;
		}
		case 'v': {
			strcpy(g_device, optarg);
			break;
		}
		case 'm': {
			mode = atoi(optarg);
			break;
		}
		case 'l': {
			line = atoi(optarg);
			break;
		}
		case 'c': {
			cdrom = atoi(optarg);
			break;
		}
		case 'f': {
			sscanf(optarg, "%g", &freq);
			if( freq != 0 )
				break;
		}
		case 'h':
		default: {
		    usage(argv[0]);
		}
	    }
	}
	
	if( line == -1 && mode == -1 && cdrom == -1 && freq == -1 ) {
		usage(argv[0]);
	}

	printf("dev:%s mode:%d, freq:%f, cdrom:%d, line:%d\n",g_device,mode,freq,cdrom,line);

	vid = open(g_device, O_NONCAP);
	if(vid < 0) {
		printf("No such video device %s\n.", g_device);
		return 1;
	}
	ioctl(vid, VIDIOC_QUERYCAP, &caps);
	
	if( 0 != strcmp( caps.name, MXB_IDENTIFIER )) {
		printf("Device \"%s\" is not a %s\n.", g_device, MXB_IDENTIFIER);
		return 1;
	}

	if( cdrom > -1 ) {
		if( cdrom < MXB_AUDIOS) {
			if( 0 != ioctl(vid, MXB_S_AUDIO_CD, &cdrom)) {
				printf("Failed to set Input-to-CD-Rom-out to %d\n",cdrom);	
				cdrom = -1;
			}
		} else {
			printf("Invalid input for cdrom-out selected.\n");	
			cdrom = -1;
		}
	}
	
	if( line > -1 ) {
		if( line < MXB_AUDIOS) {
			if( 0 != ioctl(vid, MXB_S_AUDIO_LINE, &line)) {
				printf("Failed to set Input-to-Line-out to %d\n",line);	
				line = -1;
			}
		} else {
			printf("Invalid input for line-out selected.\n");	
			line = -1;
		}
	}
	
	if( mode > -1 ) {
		struct v4l2_tuner t;
		t.input = 0;
		t.audmode = mode;
	
		if( 0 != ioctl(vid, VIDIOC_S_TUNER, &t)) {
			printf("Failed to set mode to %d\n",line);	
			mode = -1;
		}
	}
	
	if( freq > -1) {
		unsigned long set_freq = freq*16;
		if( 0 != ioctl(vid, VIDIOC_S_FREQ, &set_freq)) {
			printf("Failed to set frequency to %f\n",freq);	
			freq = -1;
		}
	}

	if(silent == 0) {

		if( line == -1 && mode == -1 && cdrom == -1 && freq == -1 ) {
		} else {
			printf("Setting '%s' ", g_device); 
			if( cdrom != -1) {
				printf("cdrom to '%s', ",mxb_audios[cdrom].name);
			}
			if( line != -1) {
				printf("line to '%s', ",mxb_audios[line].name);
			}
			if( freq != -1) {
				printf("frequency to '%f', ",freq);
			}
			if( mode != -1) {
				printf("mode to '%d', ",mode);
			}
			printf("\b\b.\n");
		}
	}
	close(vid);

	return 0;
}
