#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <string.h>

#include <stdio.h>
#include <assert.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <unistd.h>
#include <fcntl.h>


#include <linux/types.h>
#include <linux/i2c.h>

#include "videodevX/videodev.h"
#include "extensions/mxb_v4l2.h"

#define DEVICE "/dev/video0"
char g_device[64];

#define NUM_BUF	3
struct buffer {
	struct v4l2_buffer vidbuf;
	char* data;
	int length;
};

struct v4l2_requestbuffers req;
struct buffer cap_bufs[NUM_BUF];

int start_grab_stop(int fd, int count)
{
	int type = V4L2_BUF_TYPE_CAPTURE;
	fd_set rdset;
	struct timeval timeout;
	struct v4l2_buffer cur;

	int err = 0, i = 0, n = 0;
	
	for (i = 0; i < req.count; i++) {
		struct v4l2_buffer *vidbuf = &cap_bufs[i].vidbuf;
		err = ioctl(fd, VIDIOC_QBUF, vidbuf);
		if (err) {
			perror("VIDIOC_QBUF");
			return -1;
		}
	}

	err = ioctl (fd, VIDIOC_STREAMON, &type);
	if (err) {
		perror("VIDIOC_STREAMON");
		return -1;
	}

	cur.type = type;

	for(i = 0; i < count; i++) {

		FD_ZERO (&rdset);
		FD_SET (fd, &rdset);

		timeout.tv_sec = 1;
		timeout.tv_usec = 0;

		n = select (fd + 1, &rdset, NULL, NULL, &timeout);
		if (n == -1) {
			printf("select() error.\n");
			return -1;
		}
		else if (n == 0) {
			printf("select() timeout\n");
			return -1;
		}
		else if (FD_ISSET (fd, &rdset)) {
			err = ioctl(fd, VIDIOC_DQBUF, &cur);
			if (err) {
				perror("VIDIOC_DQBUF");
				return -1;
			}
		}
		printf("dequeed %d\n",cur.index);

		err = ioctl (fd, VIDIOC_QBUF, &cur);
		if (err) {
			perror(" VIDIOC_QBUF");
			return -1;
		}

		printf("queued  %d\n",cur.index);
	}

	err = ioctl (fd, VIDIOC_STREAMOFF, &type);
	if (err) {
		perror("VIDIOC_STREAMOFF");
		return -1;
	}
}


int main(int argc, char** argv)
{
	struct v4l2_buffer vidbuf;
	struct v4l2_capability	caps;
	struct v4l2_standard std;
	struct v4l2_format fmt;

	int vid = 0, err = 0;
	int width = 320, height = 240;
	
	int i = 0;
	
	strcpy(g_device, DEVICE);

	vid = open(g_device, O_RDONLY);
	if(vid < 0) {
		printf("No such video device %s\n.", g_device);
		return -1;
	}
	ioctl(vid, VIDIOC_QUERYCAP, &caps);
	
	if( 0 != strcmp( caps.name, MXB_IDENTIFIER )) {
		printf("Device \"%s\" is not a %s\n.", g_device, MXB_IDENTIFIER);
		return -1;
	}

	if(caps.type != V4L2_TYPE_CAPTURE) {
		printf("%s is not a capture device.\n", caps.name);
		return -1;
	}
	if(!(caps.flags & V4L2_FLAG_STREAMING)) {
		printf("device '%s' does not support streaming capture.\n", caps.name);
		return -1;
	}

	/* set standard */
	memset(&std, 0x0, sizeof(std));
	std.colorstandard = V4L2_COLOR_STD_PAL;
	std.colorstandard_data.pal.colorsubcarrier = V4L2_COLOR_SUBC_PAL;
	std.framerate.numerator = 1;
	std.framerate.denominator = 25;
	std.framelines = 625;
	err = ioctl(vid, VIDIOC_S_STD, &std);
	if(0 != err) {
		perror("VIDIOC_S_STD");
		return -1;
	}

	fmt.type = V4L2_BUF_TYPE_CAPTURE;
	fmt.fmt.pix.width = width;
	fmt.fmt.pix.height = height;
	fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_BGR32;
	
	err = ioctl(vid, VIDIOC_S_FMT, &fmt);
	if(err) {
   		perror("VIDIOC_S_FMT");
    		return -1;
	}

	/* Ask Video Device for Buffers */
	req.count = NUM_BUF;
     	req.type = V4L2_BUF_TYPE_CAPTURE;
	err = ioctl(vid, VIDIOC_REQBUFS, &req);
	if (err < 0 ||req.count < 1) {
		perror("VIDIOC_REQBUFS\n");
		return -1;
	}
	
	/* Query each buffer and map it to the video device */
	for (i = 0; i < req.count; i++) {
		struct v4l2_buffer *vidbuf = &cap_bufs[i].vidbuf;
      
		vidbuf->index = i;
		vidbuf->type = V4L2_BUF_TYPE_CAPTURE;
		err = ioctl(vid, VIDIOC_QUERYBUF, vidbuf);
		if (err < 0) {
			perror("VIDIOC_QUERYBUF\n");
			return -1;
		}

		cap_bufs[i].length = 0;
		cap_bufs[i].data = mmap (0, vidbuf->length, 
			PROT_READ|PROT_WRITE, MAP_PRIVATE, // MAP_SHARED,
			vid, vidbuf->offset);
		if ((int)cap_bufs[i].data == -1) {
			perror("mmap()");
			return -1;
		}
		cap_bufs[i].length = vidbuf->length; 
		printf("buffer %d: %d\n",i,vidbuf->length);
	}

	start_grab_stop(vid, 3);

	fmt.fmt.pix.width = 768;
	fmt.fmt.pix.height = 576;
	
	err = ioctl(vid, VIDIOC_S_FMT, &fmt);
	if(err) {
   		perror("VIDIOC_S_FMT");
    		return -1;
	}

	start_grab_stop(vid, 3);

	return 0;
}

#ifdef axa
	int mode = -1, line = -1, cdrom = -1, silent = 0; 
	float freq = -1;
	
	int c;


	/* parse options */
	for(;;) {
		if (-1 == (c = getopt(argc, argv, "v:m:l:c:f:hs"))) {
			break;
		}
	switch (c) {
		case 's': {
			silent = 1;
			break;
		}
		case 'v': {
			strcpy(g_device, optarg);
			break;
		}
		case 'm': {
			mode = atoi(optarg);
			break;
		}
		case 'l': {
			line = atoi(optarg);
			break;
		}
		case 'c': {
			cdrom = atoi(optarg);
			break;
		}
		case 'f': {
			sscanf(optarg, "%g", &freq);
			if( freq != 0 )
				break;
		}
		case 'h':
		default: {
		    usage(argv[0]);
		}
	    }
	}
	
	if( line == -1 && mode == -1 && cdrom == -1 && freq == -1 ) {
		usage(argv[0]);
	}

	printf("dev:%s mode:%d, freq:%f, cdrom:%d, line:%d\n",g_device,mode,freq,cdrom,line);


	if( cdrom > -1 ) {
		if( cdrom < MXB_AUDIOS) {
			if( 0 != ioctl(vid, MXB_S_AUDIO_CD, &cdrom)) {
				printf("Failed to set Input-to-CD-Rom-out to %d\n",cdrom);	
				cdrom = -1;
			}
		} else {
			printf("Invalid input for cdrom-out selected.\n");	
			cdrom = -1;
		}
	}
	
	if( line > -1 ) {
		if( line < MXB_AUDIOS) {
			if( 0 != ioctl(vid, MXB_S_AUDIO_LINE, &line)) {
				printf("Failed to set Input-to-Line-out to %d\n",line);	
				line = -1;
			}
		} else {
			printf("Invalid input for line-out selected.\n");	
			line = -1;
		}
	}
	
	if( mode > -1 ) {
		struct v4l2_tuner t;
		t.input = 0;
		t.audmode = mode;
	
		if( 0 != ioctl(vid, VIDIOC_S_TUNER, &t)) {
			printf("Failed to set mode to %d\n",line);	
			mode = -1;
		}
	}
	
	if( freq > -1) {
		unsigned long set_freq = freq*16;
		if( 0 != ioctl(vid, VIDIOC_S_FREQ, &set_freq)) {
			printf("Failed to set frequency to %d\n",freq);	
			freq = -1;
		}
	}

	if(silent == 0) {

		if( line == -1 && mode == -1 && cdrom == -1 && freq == -1 ) {
		} else {
			printf("Setting '%s' ", g_device); 
			if( cdrom != -1) {
				printf("cdrom to '%s', ",mxb_audios[cdrom].name);
			}
			if( line != -1) {
				printf("line to '%s', ",mxb_audios[line].name);
			}
			if( freq != -1) {
				printf("frequency to '%f', ",freq);
			}
			if( mode != -1) {
				printf("mode to '%d', ",mode);
			}
			printf("\b\b.\n");
		}
	}
	close(vid);

	return 0;
}

#endif
