#include <stdio.h>
#include <assert.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <unistd.h>
#include <fcntl.h>
#include <pthread.h>
#include <errno.h>

#include "videodevX/videodev.h"

#define DEFAULT_DEVICE "/dev/video"

int main(void)
{
	int i, fd, frame;

	struct video_capability capability;
	struct video_buffer buffer;
	struct video_mmap vmmap;
	struct video_mbuf mbuf;
	unsigned char *map;
	
	if((fd = open(DEFAULT_DEVICE,O_RDWR)) < 0) {
		perror("open");
		return -1;
	}
	
	if(ioctl(fd, VIDIOCGCAP, &capability) < 0) {
		perror("VIDIOCGCAP");
		return -1;
	}

	if(ioctl(fd, VIDIOCGMBUF, &mbuf)<0) {
		perror("VIDIOCGMBUF");
		return -1;
	}

	printf("size:%d, frames:%d ",mbuf.size, mbuf.frames);
	for(i = 0; i < mbuf.frames;i++) {
		printf(", offset %d: %d", i, mbuf.offsets[i]);
	}
	printf("\n");
	
	if((signed int)(map = mmap(0, mbuf.size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0)) < 0) {
		perror("mmap");
		return -1;
	}

	vmmap.frame = 0;
	vmmap.height = 288;
	vmmap.width = 352;
	vmmap.format = VIDEO_PALETTE_RGB24;
	if(ioctl(fd, VIDIOCMCAPTURE, &vmmap) < 0) {
		perror("VIDIOCMCAPTURE");
		return -1;
	}

	frame = 0;
	if(ioctl(fd, VIDIOCSYNC, &frame) < 0) {
		perror("v4lsync:VIDIOCSYNC");
		return -1;
	}

	if(munmap(map, mbuf.size) < 0) {
		perror("munmap");
		return -1;
	}

	close(fd);

	return 0;
}
