#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <curses.h>

#include <sys/time.h>

#include <linux/ioctl.h>
#include <linux/types.h>
#include "v4l2/videodev.h"

/*  set this according to your set-up */
#define DEVICE "/dev/vbi0"

int main(int argc, char** argv)
{		
	long bytes = 0;
	
	int fd = 0;
	int j = 0;

	struct v4l2_capability capability;
	struct v4l2_format format;

	fd_set rdset;
	
	unsigned char *c;
	unsigned char *p;
	unsigned char s[256];
	FILE* f;
	
	fd = open(DEVICE, O_RDONLY);
	if(fd < 0) {
		printf("No such video DEVICE %s\n.", DEVICE);
		return 1;
	}

	if( 0 != ioctl(fd, VIDIOC_QUERYCAP,&capability)) {
		printf("Error querying capabilites.\n");
		return 1;
	}

	if( 0 == (V4L2_TYPE_VBI & capability.type)) {
		printf("Device is not of type VBI.\n");
		return 1;
	}
	
	if( 0 == ((V4L2_FLAG_READ | V4L2_FLAG_SELECT | V4L2_FLAG_DATA_SERVICE) & capability.flags)) {
		printf("Device does not support needed things.\n");
		return 1;
	}

	printf("Device is '%s'\n", capability.name);

	memset(&format,0x0,sizeof(struct v4l2_format));
	if( 0 != ioctl(fd, VIDIOC_G_FMT,&format)) {
		printf("Error calling ioctl.\n");
		return 1;
	}

	/* fixme: check format here */
	printf("sampling rate: %d\n", format.fmt.vbi.sampling_rate);
	printf("offset:        %d\n", format.fmt.vbi.offset);
	printf("samples / line:%d\n", format.fmt.vbi.samples_per_line);
	printf("sample format: %d\n", format.fmt.vbi.sample_format);
	printf("flags:         %d\n", format.fmt.vbi.flags);
	printf("count:         %d/%d\n", format.fmt.vbi.count[0],format.fmt.vbi.count[1]);
	printf("start:         %d/%d\n", format.fmt.vbi.start[0],format.fmt.vbi.start[1]);

	bytes = format.fmt.vbi.samples_per_line*(format.fmt.vbi.count[0]+format.fmt.vbi.count[1]); 
	printf("Capturing %d samples per line from %d lines (%li bytes).\n",format.fmt.vbi.samples_per_line,format.fmt.vbi.count[0]+format.fmt.vbi.count[1],bytes);
	
	c = (unsigned char*)malloc(sizeof(unsigned char)*bytes);
	if( 0 == c ) {
		printf("Not enough memory.\n");
		return 1;
	}
	memset(c,0x00,sizeof(unsigned char)*bytes);

	/* wait for the next frame (no timeout) */	
	FD_ZERO(&rdset);
	FD_SET(fd, &rdset);
    	if( 0 > select(fd+1, &rdset, NULL, NULL, NULL)) {
		printf("Cannot wait for next frame.\n");
		return 1;
	}
		
	if(bytes != read(fd, c, bytes)) {
		printf("Cannot read from device.\n");
		return 1;
	}

#define TRESH 0x74
	p = c;
	for(j = 0; j < bytes; j++) {
		if(*p > TRESH)
			*p++ = 0xff;
		else
			*p++ = 0x00;
	}
	
	j = 0;
	sprintf(&s[0],"%02d.bin",j);
	f = fopen(s,"w+");
	if( NULL == f ) {
		printf("Cannot open file.\n");
		return 1;
	}
	if( 1 != fwrite(c, bytes,1,f)) {
		printf("Cannot write file.\n");
		return 1;
	}
	fclose(f);

	close(fd);

	return 0;
}
